@@ -1,13 +1,13 @@
 package BSON;
 
-use 5.008;
+use 5.010;
 use strict;
 use warnings;
 
 use base 'Exporter';
 our @EXPORT_OK = qw/encode decode/;
 
-our $VERSION = 0.03;
+our $VERSION = 0.05;
 
 use Carp;
 use Tie::IxHash;
@@ -20,17 +20,20 @@ use BSON::Binary;
 use BSON::ObjectId;
 use BSON::Code;
 use BSON::Bool;
+use BSON::String;
 
 # Maximum size of a BSON record
 our $MAX_SIZE = 16 * 1024 * 1024;
 
+# Max integer sizes
+our $min_int_32 = -2**32 / 2;
+our $max_int_32 = 2**32 / 2 - 1;
+our $min_int_64 = -2**64 / 2;
+our $max_int_64 = 2**64 / 2 - 1;
+
 #<<<
 my $int_re     = qr/^(?:(?:[+-]?)(?:[0123456789]+))$/;
 my $doub_re    = qr/^(?:(?i)(?:[+-]?)(?:(?=[0123456789]|[.])(?:[0123456789]*)(?:(?:[.])(?:[0123456789]{0,}))?)(?:(?:[E])(?:(?:[+-]?)(?:[0123456789]+))|))$/;
-my $min_int_32 = -2**32 / 2;
-my $max_int_32 = 2**32 / 2 - 1;
-my $min_int_64 = -2**64 / 2;
-my $max_int_64 = 2**64 / 2 - 1;
 #>>>
 
 sub e_name {
@@ -52,7 +55,7 @@ sub s_arr {
 sub s_int {
     my ( $key, $value ) = @_;
     if ( $value > $max_int_64 || $value < $min_int_64 ) {
-        confess("MongoDB can only handle 8-byte integers");
+        croak("MongoDB can only handle 8-byte integers");
     }
     return $value > $max_int_32 || $value < $min_int_32
       ? e_name( 0x12, $key ) . pack( 'q', $value )
@@ -160,6 +163,11 @@ sub s_hash {
             $bson .= e_name( 0x08, $key ) . ( $value ? "\1" : "\0" );
         }
 
+        # String (explicit)
+        elsif ( ref $value eq 'BSON::String' ) {
+            $bson .= e_name( 0x02, $key ) . string($value);
+        }
+
         # Int (32 and 64)
         elsif ( $value =~ $int_re ) {
             $bson .= s_int( $key, $value );
@@ -179,10 +187,9 @@ sub s_hash {
 }
 
 sub d_hash {
-    my $bson = shift;
-    my %opt  = @_;
+    my ($bson, %opt) = @_;
     my %hash = ();
-    if ( $opt{ixhash} ) { tie( %hash, 'Tie::IxHash' ) }
+    tie( %hash, 'Tie::IxHash' ) if $opt{ixhash};
     while ($bson) {
         my $value;
         ( my $type, my $key, $bson ) = unpack( 'CZ*a*', $bson );
@@ -321,19 +328,20 @@ BSON - Pure Perl implementation of MongoDB's BSON serialization
 
 =head1 VERSION
 
-Version 0.03
+Version 0.05
 
 =head1 SYNOPSIS
 
     use BSON qw/encode decode/;
 
     my $document = {
-        _id    => BSON::ObjectId->new,
-        date   => BSON::Time->new,
-        name   => 'James Bond',
-        age    => 45,
-        amount => 24587.45,
-        badass => BSON::Bool->true
+        _id      => BSON::ObjectId->new,
+        date     => BSON::Time->new,
+        name     => 'James Bond',
+        age      => 45,
+        amount   => 24587.45,
+        badass   => BSON::Bool->true,
+        password => BSON::String->new('12345')
     };
 
     my $bson = encode( $document );
@@ -344,6 +352,12 @@ Version 0.03
 This module implements BSON serialization and deserialization as described at
 L<http://bsonspec.org>. BSON is the primary data representation for MongoDB.
 
+=head1 LIMITATION
+
+MongoDB sets a limit for any BSON record to 16MB. This module does not enforce this 
+limit and you can use it to C<encode> and C<decode> structures as large as you 
+please.
+
 =head1 EXPORT
 
 The module does not export anything. You have to request C<encode> and/or
@@ -388,7 +402,7 @@ This module is thread safe.
 
 =head1 SEE ALSO
 
-L<BSON::Time>, L<BSON::ObjectId>, L<BSON::Code>,
+L<BSON::String>, L<BSON::Time>, L<BSON::ObjectId>, L<BSON::Code>,
 L<BSON::Binary>, L<BSON::Bool>, L<BSON::MinKey>, L<BSON::MaxKey>,
 L<BSON::Timestamp>, L<Tie::IxHash>, L<MongoDB>
 
